#include "dmms.h"
#include "main.h"
#include "configfile.h"

Config cfg;
GList *disabled_iplugins = NULL;
static gint ctrl_fd = 0;
static gint fd = 0;
static gint song_length=-1;

extern gchar *plugin_dir_list[];

gchar *xmms_get_gentitle_format(void) {
  return cfg.gentitle_format;
}

static void read_config(void) {
  ConfigFile *cfgfile;
  gchar *filename;

  memset(&cfg, 0, sizeof (Config));
  cfg.enabled_gplugins = NULL;
  cfg.disabled_iplugins = NULL;

  //filename = g_strconcat(g_get_home_dir(), "/.dmms/config", NULL);
  filename = g_strconcat(CONFIG_DIR, "/config", NULL);
  cfgfile = xmms_cfg_open_file(filename);
  if (cfgfile) {
    xmms_cfg_read_string(cfgfile, "xmms", "output_plugin", &cfg.outputplugin);
    xmms_cfg_read_string(cfgfile, "xmms", "enabled_gplugins", &cfg.enabled_gplugins);
    xmms_cfg_read_string(cfgfile, "xmms", "disabled_iplugins", &cfg.disabled_iplugins);
    xmms_cfg_read_string(cfgfile, "xmms", "generic_title_format", &cfg.gentitle_format);

    xmms_cfg_free(cfgfile);
  }
  if (cfg.gentitle_format == NULL)
    cfg.gentitle_format = g_strdup("%p - %t");
  if (cfg.outputplugin == NULL) {
#if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_MACHINE_SOUNDCARD_H)
    cfg.outputplugin = g_strdup_printf("%s/%s/libOSS.so", PLUGIN_DIR, plugin_dir_list[0]);
#elif defined(sun)
    cfg.outputplugin = g_strdup_printf("%s/%s/libSolaris.so", PLUGIN_DIR, plugin_dir_list[0]);
#else
    /*
     * FIXME: This implicitly means the output plugin that is first
     * in the alphabet will be used (usually the disk writer plugin)
     */
    cfg.outputplugin = g_strdup("");
#endif
  }
  
  g_free(filename);
}

void set_info(gchar * title, gint length, gint rate, gint freq, gint nch){
  gchar * tmp;
  
  length /= 1000;
  song_length = length;
  tmp = g_strdup_printf("INFO %s\t%d\t%d\t%d\t%d\n",title, length, rate, freq, nch);
  write(fd, tmp, strlen(tmp));
  g_free(tmp);
}

static void init_socket(void) {
  struct sockaddr_un saddr;
  
  saddr.sun_family = AF_UNIX;
  g_snprintf(saddr.sun_path, 10, "/tmp/dmms");
  if ((unlink(saddr.sun_path) == -1) && errno != ENOENT) {
    printf("setup_ctrlsocket(): Failed to unlink %s (Error: %s)",
      saddr.sun_path, strerror(errno));
  }

  if ((ctrl_fd = socket(AF_UNIX, SOCK_STREAM, 0)) != -1) {
    if (bind(ctrl_fd, (struct sockaddr *) &saddr, sizeof (saddr)) != -1) {
      listen(ctrl_fd, 100);
    }
    else {
      printf("Failed to assign %s to a socket (Error: %s)\n",
        saddr.sun_path, strerror(errno));
    }
  }
  else {
    printf("Failed to open socket: %s\n",strerror(errno));
  }
}

static void socket_loop(void) {
  fd_set set;
  struct timeval tv;
  struct sockaddr_un saddr;
  gint len,received;
  gint time=0, offset;
  gchar buf[1024]="";
  gchar * params;
  gchar * tmp;
  gchar * endofline;
  gchar * title;
  gint length;
  gboolean still_playing = FALSE;
  gboolean sent_stopped = TRUE;
  gboolean stop_received = FALSE;
  
  len = sizeof (saddr);
  for(; (fd = accept(ctrl_fd, (struct sockaddr *) &saddr, &len)); close(fd)) {
    write(fd, "HELLO\n",6);

    for(;;) {
      still_playing = FALSE;
      if(get_input_playing()) {
        time = input_get_time();
        if (time >= 0) {
          still_playing = TRUE;
          if (!get_input_paused()) {
            tmp = g_strdup_printf("TIME %4.3f\n",time/1000.0);
            write(fd, tmp, strlen(tmp));
            g_free(tmp);
          }
        }
      }
      if(!still_playing && !sent_stopped && !stop_received) {
        write(fd, "STOPPED\n", 8);
        sent_stopped = TRUE;
      }
      FD_ZERO(&set);
      FD_SET(fd, &set);
      tv.tv_sec = 0;
      tv.tv_usec = 100000;
      if (select(fd+1, &set, NULL, NULL, &tv) <= 0) 
         continue;
      received=read(fd, buf, sizeof(buf)-1);
      if (received <= 0)
         break;
      buf[received]='\0';
      if(strncmp(buf,"PLAY",4) == 0) {
        params=&buf[5];
        //*(strchr(params,'\n')) = '\0';
        endofline = strchr(params,'\n');
        if (endofline != NULL)
          *endofline = '\0';
        else
          continue;
        if(strlen(params) == 0) {
          write(fd, "ERR No filename\n",16);
          continue;
        }
        if(get_input_playing())
          input_stop();
         input_play(params);
         still_playing = TRUE;
         sent_stopped = FALSE;
         stop_received = FALSE;
         write(fd, "PLAY\n",5);
         sleep(3);
      }
      else if(strncmp(buf,"STOP",4) == 0) {
        input_stop();
        stop_received = TRUE;
        write(fd, "STOP\n",5);
      }
      else if(strncmp(buf,"PAUSE",5) == 0) {
        input_pause();
        write(fd, "PAUSE\n",7);
      }
      else if(strncmp(buf,"INFO",4) == 0) {
        params=&buf[5];
        //*(strchr(params,'\n')) = '\0';
        endofline = strchr(params,'\n');
        if (endofline != NULL)
          *endofline = '\0';
        else
          continue;
        if(strlen(params) == 0) {
          write(fd, "ERR No filename\n",16);
          continue;
        }
        //title = (gchar *)g_malloc(1024);
        input_get_song_info(params,&title, &length);
        tmp = g_strdup_printf("TITLE %4.3f %s\n",length/1000.0,title);
        write(fd, tmp, strlen(tmp));
        //g_free(title);
        g_free(tmp);
      }
      else if(strncmp(buf,"FWD",3) == 0) {
        params=&buf[4];
        *(strchr(params,'\n')) = '\0';
        offset = atoi(params);
        if(strlen(params) == 0) {
          write(fd, "ERR No offset\n",16);
          continue;
        }
        if(get_input_playing()) {
          time = input_get_time()/1000;
          if (time + offset < song_length) {
            time += offset;
            input_seek(time);
          }
          else {
            input_stop();
          }
        }
      }
      else if(strncmp(buf,"BWD",3) == 0) {
        params=&buf[4];
        *(strchr(params,'\n')) = '\0';
        offset = atoi(params);
        if(strlen(params) == 0) {
          write(fd, "ERR No offset\n",16);
          continue;
        }
        if(get_input_playing()) {
          time = input_get_time()/1000;
          if (time > offset) {
            time -= offset;
          }
          else {
            time = 0;
          }
          input_seek(time);
        }
      }
      else {
        write(fd, "UNKN\n",9);
      }
    }
  }
}

int main() {
  read_config();
  init_plugins();
  init_socket();
  socket_loop();
  return 1;
}
